@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2019 Kai Pastor
+ *    Copyright 2012-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -299,12 +299,28 @@ std::unique_ptr<Symbol> Symbol::load(QXmlStreamReader& xml, const Map& map, Symb
 	auto code = symbol_element.attribute<QString>(QLatin1String("code"));
 	if (symbol_element.hasAttribute(QLatin1String("id")))
 	{
-		auto id = symbol_element.attribute<QString>(QLatin1String("id"));
-		if (symbol_dict.contains(id)) 
-			throw FileFormatException(::OpenOrienteering::ImportExport::tr("Symbol ID '%1' not unique at line %2 column %3.").arg(id).arg(xml.lineNumber()).arg(xml.columnNumber()));
+		const auto id = symbol_element.attribute<QString>(QLatin1String("id"));
 		
-		symbol_dict[id] = symbol.get();  // Will be dangling pointer when we throw an exception later
+		bool conversion_ok;
+		auto const converted_id = id.toInt(&conversion_ok);
 		
+		if (!id.isEmpty())
+		{
+			if (conversion_ok)
+			{
+				if (symbol_dict.contains(converted_id))
+					throw FileFormatException(::OpenOrienteering::ImportExport::tr("Symbol ID '%1' not unique at line %2 column %3.").arg(id).arg(xml.lineNumber()).arg(xml.columnNumber()));
+
+				symbol_dict[converted_id] = symbol.get();  // Will be dangling pointer when we throw an exception later
+			}
+			else
+			{
+				throw FileFormatException(::OpenOrienteering::ImportExport::tr("Malformed symbol ID '%1' at line %2 column %3.")
+			                              .arg(id).arg(xml.lineNumber())
+			                              .arg(xml.columnNumber()));
+			}
+		}
+
 		if (code.isEmpty())
 			code = id;
 	}
@@ -472,6 +488,9 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 	
 	// Create image
 	QImage image(side_length, side_length, QImage::Format_ARGB32_Premultiplied);
+	static auto const background = qPremultiply(qRgba(254, 254, 254, 255));
+	image.fill(background);
+	
 	QPainter painter(&image);
 	if (antialiasing)
 		painter.setRenderHint(QPainter::Antialiasing);
@@ -479,7 +498,6 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 	// Make background transparent
 	auto mode = painter.compositionMode();
 	painter.setCompositionMode(QPainter::CompositionMode_Clear);
-	painter.fillRect(image.rect(), Qt::transparent);
 	painter.setCompositionMode(mode);
 	painter.translate(0.5 * side_length, 0.5 * side_length);
 	
@@ -769,7 +787,7 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 		{
 			for (int x = image.width() - 1; x >= 0; --x)
 			{
-				if (qAlpha(image.pixel(x, y)) > 0)
+				if (image.pixel(x, y) != background)
 					continue;
 				
 				auto is_white = [](const QRgb& rgb) {
@@ -783,7 +801,7 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 					auto preceding = image.pixel(x-1, y);
 					if (is_white(preceding))
 					{
-						image.setPixel(x, y, qPremultiply(qRgba(0, 0, 0, 127)));
+						image.setPixel(x, y, qPremultiply(qRgba(192, 192, 192, 255)));
 						continue;
 					}
 				}
@@ -792,7 +810,7 @@ QImage Symbol::createIcon(const Map& map, int side_length, bool antialiasing, qr
 					auto preceding = image.pixel(x, y-1);
 					if (is_white(preceding))
 					{
-						image.setPixel(x, y, qPremultiply(qRgba(0, 0, 0, 127)));
+						image.setPixel(x, y, qPremultiply(qRgba(192, 192, 192, 255)));
 					}
 				}
 			}
@@ -821,6 +839,11 @@ qreal Symbol::calculateLargestLineExtent() const
 	return 0;
 }
 
+const Symbol::BorderHints* Symbol::borderHints() const
+{
+	return nullptr;
+}
+
 
 
 QString Symbol::getPlainTextName() const
