@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2019 Kai Pastor
+ *    Copyright 2012-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -59,7 +59,7 @@ class SymbolSettingDialog;
 class TextSymbol;
 class VirtualCoordVector;
 
-typedef QHash<QString, Symbol*> SymbolDictionary;
+typedef QHash<qint32, Symbol*> SymbolDictionary;
 
 
 /**
@@ -101,6 +101,23 @@ public:
 	Q_DECLARE_FLAGS(RenderableOptions, RenderableOption)
 	
 	
+	/**
+	 * Hints for tools that operator on border lines.
+	 */
+	struct BorderHints
+	{
+		struct SingleSide
+		{
+			bool active;
+			int join_style;
+			double main_shift;
+			double extra_shift;
+		};
+		SingleSide left;
+		SingleSide right;
+	};
+	
+	
 	explicit Symbol(Type type) noexcept;
 	virtual ~Symbol();
 	
@@ -138,6 +155,8 @@ public:
 	 * This function does not check the equality of the state
 	 * (protected/hidden). Use stateEquals(Symbol*) for this comparison.
 	 * 
+	 * It also does not compare color priorities.
+	 * 
 	 * @param other The symbol to compare with.
 	 * @param case_sensitivity Comparison mode for strings, e.g. symbol names.
 	 */
@@ -382,6 +401,13 @@ public:
 	 */
 	virtual qreal calculateLargestLineExtent() const;
 	
+	/**
+	 * Determines the border hints for this symbol.
+	 * 
+	 * The default implementation returns nullptr.
+	 */
+	virtual const BorderHints* borderHints() const;
+	
 	
 	// Getters / Setters
 	
