@@ -1,5 +1,5 @@
 /*
- *    Copyright 2013-2019 Kai Pastor
+ *    Copyright 2013-2020 Kai Pastor
  *
  *    Some parts taken from file_format_oc*d8{.h,_p.h,cpp} which are
  *    Copyright 2012 Pete Curtis
@@ -31,6 +31,7 @@
 #include <type_traits>
 #include <vector>
 
+#include <Qt>
 #include <QtGlobal>
 #include <QtMath>
 #include <QtNumeric>
@@ -38,15 +39,12 @@
 #include <QCoreApplication>
 #include <QDebug>
 #include <QDir>
-#include <QFileInfo>
 #include <QFlags>
 #include <QFontMetricsF>
 #include <QIODevice>
 #include <QImage>
-#include <QImageReader>
 #include <QLatin1Char>
 #include <QLatin1String>
-#include <QList>
 #include <QPointF>
 #include <QTextCodec>
 #include <QTextDecoder>
@@ -77,8 +75,8 @@
 #include "fileformats/ocd_types_v12.h"  // IWYU pragma: keep
 #include "fileformats/ocd_types_v2018.h"
 #include "templates/template.h"
-#include "templates/template_image.h"
 #include "templates/template_map.h"
+#include "templates/template_placeholder.h"
 #include "util/encoding.h"
 #include "util/util.h"
 
@@ -893,24 +891,10 @@ void OcdFileImport::importTemplate(const QString& param_string)
 	const QChar* unicode = param_string.unicode();
 	
 	int i = param_string.indexOf(QLatin1Char('\t'), 0);
-	const QString filename = QString::fromRawData(unicode, qMax(-1, i));
-	const QString clean_path = QDir::cleanPath(QString(filename).replace(QLatin1Char('\\'), QLatin1Char('/')));
-	const QString extension = QFileInfo(clean_path).suffix().toLower();
-	
-	Template* templ = nullptr;
-	if (extension.compare(QLatin1String("ocd")) == 0)
-	{
-		templ = new TemplateMap(clean_path, map);
-	}
-	else if (QImageReader::supportedImageFormats().contains(extension.toLatin1()))
-	{
-		templ = new TemplateImage(clean_path, map);
-	}
-	else
-	{
-		addWarning(tr("Unable to import template: \"%1\" is not a supported template type.").arg(filename));
-		return;
-	}
+	auto const filename = QString::fromRawData(unicode, qMax(-1, i)).replace(QLatin1Char('\\'), QLatin1Char('/'));
+	auto const clean_path = QDir::cleanPath(filename);
+	// Leave template type resolution to Importer::validate().
+	auto* templ = new TemplatePlaceholder("", clean_path, map);
 	
 	// 8 or 9 or 10 ? Only tested with 8 and 11
 	double scale_factor = (ocd_version <= 8) ? 0.01 : 1.0;
@@ -920,6 +904,7 @@ void OcdFileImport::importTemplate(const QString& param_string)
 	double scale_y = 1.0;
 	int dimming = 0;
 	bool visible = false;
+	bool explicit_positioning = false;
 	
 	while (i >= 0)
 	{
@@ -934,11 +919,13 @@ void OcdFileImport::importTemplate(const QString& param_string)
 			// empty item
 			break;
 		case 'x':
+			explicit_positioning = true;
 			value = param_value.toDouble(&ok);
 			if (ok)
 				templ->setTemplateX(qRound64(value*1000*scale_factor));
 			break;
 		case 'y':
+			explicit_positioning = true;
 			value = param_value.toDouble(&ok);
 			if (ok)
 				templ->setTemplateY(-qRound64(value*1000*scale_factor));
@@ -946,16 +933,19 @@ void OcdFileImport::importTemplate(const QString& param_string)
 		case 'a':
 		case 'b':
 			// TODO: use the distinct angles correctly, not just the average
+			explicit_positioning = true;
 			rotation += param_value.toDouble(&ok);
 			if (ok)
 				++num_rotation_params;
 			break;
 		case 'u':
+			explicit_positioning = true;
 			value = param_value.toDouble(&ok);
 			if (ok && qAbs(value) >= 0.0000000001)
 				scale_x = value;
 			break;
 		case 'v':
+			explicit_positioning = true;
 			value = param_value.toDouble(&ok);
 			if (ok && qAbs(value) >= 0.0000000001)
 				scale_y = value;
@@ -972,15 +962,23 @@ void OcdFileImport::importTemplate(const QString& param_string)
 		i = next_i;
 	}
 	
-	if (num_rotation_params)
-		templ->setTemplateRotation(Georeferencing::degToRad(rotation / num_rotation_params));
+	if (explicit_positioning)
+	{
+		if (num_rotation_params)
+			templ->setTemplateRotation(Georeferencing::degToRad(rotation / num_rotation_params));
+		
+		templ->setTemplateScaleX(scale_x * scale_factor);
+		templ->setTemplateScaleY(scale_y * scale_factor);
+	}
+	else if (clean_path.endsWith(QLatin1String(".ocd"), Qt::CaseInsensitive))
+	{
+		// OCD templates must use the map's scale and georeferencing.
+		// The transformation must be determined after loading the template.
+		templ->setProperty(TemplateMap::ocdTransformProperty(), true);
+	}
 	
-	templ->setTemplateScaleX(scale_x * scale_factor);
-	templ->setTemplateScaleY(scale_y * scale_factor);
-	
-	int template_pos = map->getFirstFrontTemplate();
-	map->addTemplate(templ, 0);
-	map->setFirstFrontTemplate(template_pos+1);
+	auto const template_pos = std::min(0, map->getFirstFrontTemplate() - 1);
+	map->addTemplate(template_pos, std::unique_ptr<Template>{templ});
 	
 	if (view)
 	{
@@ -1555,6 +1553,7 @@ Symbol* OcdFileImport::importAreaSymbol(const Ocd::AreaSymbolV8& ocd_symbol)
 	setupBaseSymbol(symbol, ocd_symbol.base);
 	setupAreaSymbolCommon(
 	            symbol,
+	            ocd_symbol.base.flags & Ocd::SymbolRotatable,
 	            ocd_symbol.fill_on,
 	            ocd_symbol.common,
 	            ocd_symbol.data_size,
@@ -1570,6 +1569,7 @@ Symbol* OcdFileImport::importAreaSymbol(const S& ocd_symbol)
 	setupBaseSymbol(symbol, ocd_symbol.base);
 	setupAreaSymbolCommon(
 	            symbol,
+	            ocd_symbol.base.flags & Ocd::SymbolRotatable,
 	            ocd_symbol.common.fill_on_V9,
 	            ocd_symbol.common,
 	            ocd_symbol.data_size,
@@ -1597,7 +1597,7 @@ Symbol* OcdFileImport::importAreaSymbol(const S& ocd_symbol)
 	return combined;
 }
 
-void OcdFileImport::setupAreaSymbolCommon(OcdImportedAreaSymbol* symbol, bool fill_on, const Ocd::AreaSymbolCommonV8& ocd_symbol, std::size_t data_size, const Ocd::PointSymbolElementV8* elements)
+void OcdFileImport::setupAreaSymbolCommon(OcdImportedAreaSymbol* symbol, bool rotatable, bool fill_on, const Ocd::AreaSymbolCommonV8& ocd_symbol, std::size_t data_size, const Ocd::PointSymbolElementV8* elements)
 {
 	// Basic area symbol fields: minimum_area, color
 	symbol->minimum_area = 0;
@@ -1606,12 +1606,12 @@ void OcdFileImport::setupAreaSymbolCommon(OcdImportedAreaSymbol* symbol, bool fi
 	symbol->patterns.reserve(4);
 	
 	// Hatching
-	if (ocd_symbol.hatch_mode != Ocd::HatchNone)
+	if (ocd_symbol.hatch_mode != Ocd::HatchNone && ocd_symbol.hatch_line_width)
 	{
 		AreaSymbol::FillPattern pattern;
 		pattern.type = AreaSymbol::FillPattern::LinePattern;
 		pattern.angle = convertAngle(ocd_symbol.hatch_angle_1);
-		pattern.setRotatable(true);
+		pattern.setRotatable(rotatable);
 		pattern.line_spacing = convertLength(ocd_symbol.hatch_dist);
 		pattern.line_offset = 0;
 		pattern.line_width = convertLength(ocd_symbol.hatch_line_width);
@@ -1630,12 +1630,13 @@ void OcdFileImport::setupAreaSymbolCommon(OcdImportedAreaSymbol* symbol, bool fi
 		}
 	}
 	
-	if (ocd_symbol.structure_mode != Ocd::StructureNone)
+	if (ocd_symbol.structure_mode != Ocd::StructureNone && ocd_symbol.structure_height
+	    && ocd_symbol.structure_width && data_size)
 	{
 		AreaSymbol::FillPattern pattern;
 		pattern.type = AreaSymbol::FillPattern::PointPattern;
 		pattern.angle = convertAngle(ocd_symbol.structure_angle);
-		pattern.setRotatable(true);
+		pattern.setRotatable(rotatable);
 		pattern.point_distance = convertLength(ocd_symbol.structure_width);
 		pattern.line_spacing = convertLength(ocd_symbol.structure_height);
 		pattern.line_offset = 0;
