@@ -1,5 +1,5 @@
 /*
- *    Copyright 2016-2019 Kai Pastor
+ *    Copyright 2016-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -185,6 +185,28 @@ class OgrFileImport : public Importer
 	Q_DECLARE_TR_FUNCTIONS(OpenOrienteering::OgrFileImport)
 	
 public:
+	using ObjectList = std::vector<Object*>;
+	
+	/**
+	 * An interface for clipping objects during import.
+	 */
+	class Clipping
+	{
+	public:
+		virtual ~Clipping();
+		
+		/**
+		 * Applies clipping to the input objects.
+		 * 
+		 * This function takes ownership of the input objects, and
+		 * it passes ownership of the output objects to the caller.
+		 */
+		virtual ObjectList process(const ObjectList& objects) const = 0;
+	};
+	
+	
+	static bool canRead(const QString& path);
+	
 	/**
 	 * The unit type indicates the coordinate system the data units refers to.
 	 */
@@ -246,6 +268,8 @@ protected:
 	 */
 	static bool checkGeoreferencing(OGRDataSourceH data_source, const Georeferencing& georef);
 	
+	void prepare() override;
+	
 	bool importImplementation() override;
 	
 	ogr::unique_srs importGeoreferencing(OGRDataSourceH data_source);
@@ -254,9 +278,8 @@ protected:
 	
 	void importLayer(MapPart* map_part, OGRLayerH layer);
 	
-	void importFeature(MapPart* map_part, OGRFeatureDefnH feature_definition, OGRFeatureH feature, OGRGeometryH geometry);
+	void importFeature(MapPart* map_part, OGRFeatureDefnH feature_definition, OGRFeatureH feature, OGRGeometryH geometry, const Clipping* clipping);
 	
-	using ObjectList = std::vector<Object*>;
 	
 	ObjectList importGeometry(OGRFeatureH feature, OGRGeometryH geometry);
 	
@@ -268,6 +291,11 @@ protected:
 	
 	PathObject* importPolygonGeometry(OGRFeatureH feature, OGRGeometryH geometry);
 	
+	std::unique_ptr<Clipping> getLayerClipping(OGRLayerH layer);
+	
+	
+	bool setSRS(OGRSpatialReferenceH srs);
+	
 	
 	Symbol* getSymbol(Symbol::Type type, const char* raw_style_string);
 	
@@ -336,6 +364,7 @@ private:
 	UnitType unit_type;
 	
 	bool georeferencing_import_enabled = true;
+	bool clip_layers;
 };
 
 
