@@ -22,6 +22,8 @@
 #ifndef OPENORIENTEERING_GEOREFERENCING_DIALOG_H
 #define OPENORIENTEERING_GEOREFERENCING_DIALOG_H
 
+#include <vector>
+
 #include <QDialog>
 #include <QObject>
 #include <QScopedPointer>
@@ -32,6 +34,7 @@
 
 class QAction;
 class QCursor;
+class QCheckBox;
 class QDialogButtonBox;
 class QDoubleSpinBox;
 class QLabel;
@@ -126,6 +129,16 @@ public:
 	  */
 	void declinationChanged();
 	
+	/**
+	  * Updates the scale factor widget from the georeferencing.
+	  */
+	void auxiliaryFactorChanged();
+	
+	/**
+	  * Sets visibility of scale compensation widgets.
+	  */
+	void showScaleChanged(bool checked);
+	
 	/**
 	 * Triggers an online request for the magnetic declination.
 	 * 
@@ -190,9 +203,14 @@ protected:
 	void crsEdited();
 	
 	/**
-	 * Notifies the dialog of a change in the grid scale factor.
+	 * Notifies the dialog of a change in the auxiliary scale factor.
 	 */
-	void scaleFactorEdited();
+	void auxiliaryFactorEdited(double value);
+	
+	/**
+	 * Updates the combined scale factor field from the underlying Georeferencing.
+	 */
+	void updateCombinedFactor();
 	
 	/**
 	 * Hides the dialog and activates a GeoreferencingTool for selecting
@@ -245,12 +263,12 @@ private:
 	bool tool_active;
 	bool declination_query_in_progress;
 	bool grivation_locked;
+	bool scale_factor_locked;
 	
 	/* GUI elements */
 	CRSSelector* crs_selector;
 	QLabel* status_label;
 	QLabel* status_field;
-	QDoubleSpinBox* scale_factor_edit;
 	
 	QDoubleSpinBox* map_x_edit;
 	QDoubleSpinBox* map_y_edit;
@@ -271,6 +289,11 @@ private:
 	QDoubleSpinBox* declination_edit;
 	QPushButton* declination_button;
 	QLabel* grivation_label;
+
+	QCheckBox* show_scale_check;
+	std::vector<QWidget*> scale_widget_list;
+	QDoubleSpinBox* scale_factor_edit;
+	QLabel* combined_factor_display;
 	
 	QDialogButtonBox* buttons_box;
 	QPushButton* reset_button;
