@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013, 2014 Thomas Schöps
- *    Copyright 2013-2017 Kai Pastor
+ *    Copyright 2013-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -61,18 +61,19 @@ class GPSTemporaryMarkers;
 class GPSTrackRecorder;
 class GeoreferencingDialog;
 class MainWindow;
+class MapCoordF;
 class MapEditorActivity;
 class MapEditorTool;
 class MapFindFeature;
 class MapView;
 class MapWidget;
+class PaintOnTemplateFeature;
 class PrintWidget;
 class ReopenTemplateDialog;
 class Symbol;
 class SymbolWidget;
 class Template;
 class TemplateListWidget;
-class TemplatePositionDockWidget;
 
 
 /**
@@ -93,6 +94,13 @@ public:
 		SymbolEditor = 1
 	};
 	
+	/** Locations where to place popup windows. */
+	enum PopupLocation
+	{
+		PopupLocationTop = 0x04,
+		PopupLocationBottom = 0x08,
+	};
+	
 	/**
 	 * Constructs a new MapEditorController for a map.
 	 * 
@@ -181,7 +189,10 @@ public:
 	
 	/**
 	 * Sets the current editor activity.
-	 * new_activity may be nullptr to disable the current editor activity.
+	 * 
+	 * Before setting the new activity, this function deletes the previous
+	 * activity. Parameter new_activity may be nullptr to just terminate the
+	 * current editor activity.
 	 */
 	void setEditorActivity(MapEditorActivity* new_activity);
 	
@@ -195,20 +206,6 @@ public:
 	/** Returns this controller's symbol widget, where the symbol selection happens. */
 	inline SymbolWidget* getSymbolWidget() const {return symbol_widget;}
 	
-	/** Returns if a template position dock widget exists for a template. */
-	inline bool existsTemplatePositionDockWidget(Template* temp) const {return template_position_widgets.contains(temp);}
-	/** Returns the template position dock widget for a template. */
-	inline TemplatePositionDockWidget* getTemplatePositionDockWidget(Template* temp) const {return template_position_widgets.value(temp);}
-	/** Adds a template position dock widget for the given template. */
-	void addTemplatePositionDockWidget(Template* temp);
-	/**
-	 * Removes the template position dock widget for the template.
-	 * 
-	 * Should be called by the dock widget if it is closed or the
-	 * template deleted; deletes the dock widget.
-	 */
-	void removeTemplatePositionDockWidget(Template* temp);
-	
 	
 	/**
 	 * Shows the given widget in a popup window with specified title.
@@ -219,7 +216,7 @@ public:
 	 * 
 	 * Make sure that the child widget has a reasonable size hint.
 	 */
-	void showPopupWidget(QWidget* child_widget, const QString& title);
+	void showPopupWidget(QWidget* child_widget, const QString& title, PopupLocation location = PopupLocationBottom);
 	
 	/**
 	 * Deletes the given popup widget, which was previously shown with
@@ -292,6 +289,10 @@ public slots:
 	void pan();
 	/** Moves view to GPS position. */
 	void moveToGpsPos();
+	/** Activates or stops follow-position mode. */
+	void followPositionClicked(bool enable);
+	/** Follow-position mode update handler. */
+	void followPositionUpdate(OpenOrienteering::MapCoordF position);
 	/** Zooms in in the current map widget. */
 	void zoomIn();
 	/** Zooms out in the current map widget. */
@@ -464,11 +465,6 @@ public slots:
 	 *  The prerequisites for using the tool must be given. */
 	void distributePointsClicked();
 	
-	/** Shows or hides the paint-on-template widget */
-	void paintOnTemplateClicked(bool checked);
-	/** Shows the template selection dialog for for the paint-on-template functionality */
-	void paintOnTemplateSelectClicked();
-	
 	/** Enables or disables GPS display. */
 	void enableGPSDisplay(bool enable);
 	/** Enables or disables showing distance rings when GPS display is active. */
@@ -568,23 +564,26 @@ public slots:
 	void setViewOptionsEnabled(bool enabled = true);
 	
 	/**
-	 * Indicates a change of the current toolbar and dock widget positions,
-	 * and schedules saving.
+	 * Indicates a change of the current toolbar and dock widget visibilities
+	 * and locations, and schedules saving.
 	 */
 	void setWindowStateChanged();
 	
 private:
 	/**
-	 * Immediately saves the window state if needed.
+	 * Saves the window state in the permanent settings.
 	 * 
-	 * This will save the current toolbar and dock widget positions if the
-	 * window state is marked as changed.
-	 * After saving, it marks the state as clean.
+	 * The window state consists of current toolbar and dock widget visibility
+	 * and locations.
+	 * 
+	 * This function does nothing in mobile mode or symbol editor mode.
 	 */
 	void saveWindowState();
 	
 	/**
 	 * Restores previously saved toolbar and dock widget positions.
+	 * 
+	 * This function does nothing in mobile mode or symbol editor mode.
 	 */
 	void restoreWindowState();
 	
@@ -663,9 +662,6 @@ private:
 	void createMenuAndToolbars();
 	void createMobileGUI();
 	
-	void paintOnTemplate(Template* temp);
-	void finishPaintOnTemplate();
-	
 	void doUndo(bool redo);
 	
 	Map* map;
@@ -710,6 +706,7 @@ private:
 	
 	QAction* pan_act;
 	QAction* move_to_gps_pos_act;
+	QAction* follow_position_act;
 	QAction* zoom_in_act;
 	QAction* zoom_out_act;
 	QAction* show_all_act;
@@ -790,9 +787,7 @@ private:
 	QAction* cutaway_physical_act;
 	QAction* distribute_points_act;
 	
-	QAction* paint_on_template_act;
-	QAction* paint_on_template_settings_act;
-	Template* last_painted_on_template;
+	std::unique_ptr<PaintOnTemplateFeature> paint_feature;
 	
 	QAction* touch_cursor_action;
 	QAction* gps_display_action;
@@ -829,8 +824,8 @@ private:
 	QToolBar* toolbar_mapparts = nullptr;
 	
 	// For mobile UI
-	ActionGridBar* bottom_action_bar;
-	ActionGridBar* top_action_bar;
+	ActionGridBar* bottom_action_bar = nullptr;
+	ActionGridBar* top_action_bar = nullptr;
 	QToolButton* show_top_bar_button;
 	QAction* mobile_symbol_selector_action;
 	QMenu* mobile_symbol_button_menu;
@@ -840,8 +835,6 @@ private:
 	QScopedPointer<GeoreferencingDialog> georeferencing_dialog;
 	QScopedPointer<ReopenTemplateDialog> reopen_template_dialog;
 	
-	QHash<Template*, TemplatePositionDockWidget*> template_position_widgets;
-
 	QSignalMapper* mappart_merge_mapper;
 	QSignalMapper* mappart_move_mapper;
 };
