@@ -23,14 +23,12 @@
 
 #include <algorithm>
 
-#include <Qt>
 #include <QtGlobal>
 #include <QAction>
 #include <QGridLayout>
 #include <QIcon>
 #include <QLayout>
 #include <QMenu>
-#include <QPixmap>
 #include <QPoint>
 #include <QResizeEvent>
 #include <QSizePolicy>
@@ -74,27 +72,11 @@ int ActionGridBar::columnCount() const
 
 void ActionGridBar::addAction(QAction* action, int row, int col, int row_span, int col_span, bool at_end)
 {
-	// Determine icon size (important for high-dpi screens).
-	// Use a somewhat smaller size than what would cover the whole icon to
-	// account for the (assumed) button border.
-	QSize icon_size = getIconSize(row_span, col_span);
-	
-	// Ensure that the icon of the given action is big enough. If not, scale it up.
-	// NOTE: Here, row_span == col_span is assumed.
-	QIcon icon = action->icon();
-	QPixmap pixmap = icon.pixmap(icon_size, QIcon::Normal, QIcon::Off);
-	if (! pixmap.isNull() && pixmap.width() < icon_size.width())
-	{
-		pixmap = pixmap.scaled(icon_size, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-		icon.addPixmap(pixmap);
-		action->setIcon(icon);
-	}
-
 	auto* button = new QToolButton();
 	button->setDefaultAction(action);
 	button->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 	button->setAutoRaise(true);
-	button->setIconSize(icon_size);
+	button->setIconSize(getIconSize(row_span, col_span));
 	
 	// Add the item
 	items.push_back({action, button, next_id++, row, col, row_span, col_span, at_end});
@@ -150,8 +132,18 @@ ActionGridBar::ButtonDisplay ActionGridBar::buttonDisplay(const QToolButton* but
 
 QSize ActionGridBar::sizeHint() const
 {
-	auto extent = row_count * button_size_px;
-	return {extent, extent};
+	int actions_at_begin = -1;
+	int actions_at_end = -1;
+	for (auto const& item : items)
+	{
+		if (item.at_end)
+			actions_at_end = std::max(actions_at_end, item.col);
+		else
+			actions_at_begin = std::max(actions_at_begin, item.col);
+	}
+	auto const dynamic_size = (actions_at_begin + actions_at_end + 2) * button_size_px;
+	auto const fixed_size = rowCount() * button_size_px;
+	return direction == Horizontal ? QSize{ dynamic_size, fixed_size } : QSize{ fixed_size, dynamic_size };
 }
 
 void ActionGridBar::overflowActionClicked()
