@@ -97,7 +97,7 @@ SymbolRenderWidget::SymbolRenderWidget(Map* map, bool mobile_mode, QWidget* pare
 	
 	QShortcut* description_shortcut = new OverridingShortcut(
 	  QKeySequence(tr("F1", "Shortcut for displaying the symbol's description")),
-	  window()
+	  this
 	);
 	tooltip = new SymbolToolTip(this, description_shortcut);
 	// TODO: Use a placeholder in the literal and pass the actual shortcut's string representation.
@@ -935,17 +935,17 @@ void SymbolRenderWidget::pasteSymbols()
 		///       See also MapEditor::importMap.
 		int answer = QMessageBox::question(
 		                 window(),
-		                 Map::tr("Question"),
-		                 Map::tr("The scale of the imported data is 1:%1 "
-		                         "which is different from this map's scale of 1:%2.\n\n"
-		                         "Rescale the imported data?")
+		                 ::OpenOrienteering::Map::tr("Question"),
+		                 ::OpenOrienteering::Map::tr("The scale of the imported data is 1:%1 "
+		                                             "which is different from this map's scale of 1:%2.\n\n"
+		                                             "Rescale the imported data?")
 		                 .arg(QLocale().toString(paste_map.getScaleDenominator()),
 		                      QLocale().toString(map->getScaleDenominator())),
 		                 QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel, QMessageBox::Yes);
 		if (answer == QMessageBox::Cancel)
 			return;
 		
-		paste_map.changeScale(map->getScaleDenominator(), {0, 0}, answer == QMessageBox::Yes, false, false, false);
+		paste_map.changeScale(map->getScaleDenominator(), 1.0, {0, 0}, answer == QMessageBox::Yes, false, false, false);
 	}
 	
 	// Ensure that a change in selection is detected
