@@ -1,6 +1,6 @@
 /*
  *    Copyright 2012, 2013 Thomas Schöps
- *    Copyright 2012-2019 Kai Pastor
+ *    Copyright 2012-2020 Kai Pastor
  *
  *    This file is part of OpenOrienteering.
  *
@@ -32,11 +32,13 @@
 #include <QtTest>
 #include <QBuffer>
 #include <QByteArray>
+#include <QChar>
 #include <QCoreApplication>
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QIODevice>
+#include <QLatin1Char>
 #include <QLatin1String>
 #include <QPageSize>
 #include <QPoint>
@@ -60,6 +62,7 @@
 #include "core/map_printer.h"
 #include "core/objects/object.h"
 #include "core/objects/text_object.h"
+#include "core/symbols/area_symbol.h"
 #include "core/symbols/symbol.h"
 #include "fileformats/file_format.h"
 #include "fileformats/file_format_registry.h"
@@ -128,6 +131,49 @@ namespace QTest
 
 namespace
 {
+	/**
+	 * Provides QCOMPARE-style symbol property comparison.
+	 * 
+	 * This macro reports the symbol, but avoids expensive string operations
+	 * when the properties match.
+	 */
+	#define COMPARE_SYMBOL_PROPERTY(a, b, symbol) \
+	if ((a) != (b)) \
+	{ \
+		auto const diff = qstrlen(#b) - qstrlen(#a); \
+		auto const fill_a = QString().fill(QChar::Space, +diff); \
+		auto const fill_b = QString().fill(QChar::Space, -diff); \
+		QFAIL(QString::fromLatin1( \
+		       "Compared values are not the same (%1 %2)\n   Actual   (%3)%4: %7\n   Expected (%5)%6: %8") \
+		      .arg((symbol).getNumberAsString(), (symbol).getPlainTextName(), \
+		           QString::fromUtf8(#a), fill_a, \
+		           QString::fromUtf8(#b), fill_b) \
+		      .arg(a).arg(b) \
+		      .toUtf8()); \
+	} \
+	else \
+	{ \
+		QVERIFY(true);  /* for QEXPECT_FAIL etc. */ \
+	}
+	
+	/**
+	 * Provides QVERIFY-style symbol property verification.
+	 * 
+	 * This macro reports the symbol, but avoids expensive string operations
+	 * when the properties match.
+	 */
+	#define VERIFY_SYMBOL_PROPERTY(cond, symbol) \
+	if (cond) \
+	{ \
+		QVERIFY2(cond, QByteArray((symbol).getNumberAsString().toUtf8() + ' ' \
+		                          + (symbol).getPlainTextName().toUtf8())); \
+	} \
+	else \
+	{ \
+		QVERIFY(true);  /* for QEXPECT_FAIL etc. */ \
+	}
+	
+	
 	void comparePrinterConfig(const MapPrinterConfig& copy, const MapPrinterConfig& orig)
 	{
 		QCOMPARE(copy.center_print_area, orig.center_print_area);
@@ -140,6 +186,14 @@ namespace
 		QCOMPARE(copy.single_page_print_area, orig.single_page_print_area);
 	}
 	
+	void compareSymbol(const Symbol& actual, const Symbol& expected)
+	{
+		COMPARE_SYMBOL_PROPERTY(actual.isHidden(), expected.isHidden(), expected);
+		COMPARE_SYMBOL_PROPERTY(actual.isProtected(), expected.isProtected(), expected);
+		VERIFY_SYMBOL_PROPERTY(actual.stateEquals(&expected), expected);
+		VERIFY_SYMBOL_PROPERTY(actual.equals(&expected, Qt::CaseInsensitive), expected);
+	}
+	
 	void compareMaps(const Map& actual, const Map& expected)
 	{
 		// TODO: This does not compare everything - yet ...
@@ -186,11 +240,7 @@ namespace
 		}
 		else for (int i = 0; i < actual.getNumSymbols(); ++i)
 		{
-			if (!actual.getSymbol(i)->equals(expected.getSymbol(i), Qt::CaseSensitive))
-				qDebug("%s vs %s", qPrintable(actual.getSymbol(i)->getNumberAsString()),
-				                   qPrintable(expected.getSymbol(i)->getNumberAsString()));
-			QVERIFY(actual.getSymbol(i)->equals(expected.getSymbol(i), Qt::CaseSensitive));
-			QVERIFY(actual.getSymbol(i)->stateEquals(expected.getSymbol(i)));
+			compareSymbol(*actual.getSymbol(i), *expected.getSymbol(i));
 		}
 		
 		// Parts and objects
@@ -256,6 +306,100 @@ namespace
 	}
 	
 	
+	void fuzzyCompareSymbol(const AreaSymbol& actual, const AreaSymbol& expected)
+	{
+		auto pattern_rotable = false;
+		for (auto i = 0; i < expected.getNumFillPatterns(); ++i)
+			pattern_rotable |= expected.getFillPattern(i).rotatable();
+		for (auto i = 0; i < actual.getNumFillPatterns(); ++i)
+			COMPARE_SYMBOL_PROPERTY(actual.getFillPattern(i).rotatable(), pattern_rotable, expected);
+	}
+	
+	void fuzzyCompareSymbol(const Symbol& actual, const Symbol& expected, const QByteArray& format_id)
+	{
+		COMPARE_SYMBOL_PROPERTY(actual.isHidden(), expected.isHidden(), expected);
+		COMPARE_SYMBOL_PROPERTY(actual.isProtected(), expected.isProtected(), expected);
+		
+		if (format_id == "OCD-legacy")
+			return;  // Unmaintained, no other properties testedd.
+		
+		COMPARE_SYMBOL_PROPERTY(actual.isRotatable(), expected.isRotatable(), expected);
+		
+		/// \todo Ideally, the dominant color would be preserved.
+#ifdef EXPORT_PRESERVES_DOMINANT_COLOR
+		COMPARE_SYMBOL_PROPERTY(actual.guessDominantColor()->getName(), expected.guessDominantColor()->getName(), expected);
+#endif
+		if (actual.getType() != expected.getType())
+			return;  // The following tests assume the same type.
+		
+		switch (actual.getType())
+		{
+		case Symbol::Area:
+			fuzzyCompareSymbol(static_cast<AreaSymbol const&>(actual), static_cast<AreaSymbol const&>(expected));
+			break;
+			
+		default:
+			;  /// \todo Extend fuzzy testing
+		}
+		
+	}
+	
+	void fuzzyMatchSymbol(const Map& map, const QByteArray& format_id, const Symbol* expected)
+	{
+		Symbol const* actual = nullptr;
+		for (auto i = 0; !actual && i < map.getNumSymbols(); ++i)
+		{
+			auto const* symbol = map.getSymbol(i);
+			if (symbol->getType() != expected->getType()
+			    && symbol->getType() != Symbol::Combined)
+			{
+				continue;
+			}
+			
+			if (format_id.startsWith("OCD"))
+			{
+				// In OCD format, export may have incremented second and or first component.
+				if (symbol->getNumberComponent(0) < expected->getNumberComponent(0)
+				    || symbol->getNumberComponent(0) > expected->getNumberComponent(0) + 1)
+				{
+					continue;
+				}
+			}
+			else if (expected->getNumberAsString() != symbol->getNumberAsString())
+			{
+				continue;
+			}
+			
+			if (!expected->getPlainTextName().startsWith(symbol->getPlainTextName()))
+				continue;
+			
+			actual = symbol;
+		}
+		if (format_id == "OCD8" || format_id == "OCD-legacy")
+		{
+			// Don't elaborate testing for these legacy formats.
+			switch (expected->getType())
+			{
+			case Symbol::Combined:
+				// Expected symbol may be entirely dropped. (Its parts live independently.)
+				if (!actual)
+					return;
+				break;
+			case Symbol::Line:
+				// Expected symbol may be entirely turned into combined symbol.
+				if (!actual && format_id == "OCD-legacy")
+					return;
+				break;
+			default:
+				;  // nothing
+			}
+		}
+		if (actual)
+			fuzzyCompareSymbol(*actual, *expected, format_id);
+		else
+			QFAIL(qPrintable(QString::fromLatin1("Missing symbol: %1 %2").arg(expected->getNumberAsString(), expected->getPlainTextName())));
+	}
+	
 	/**
 	 * Compares map features in a way that works for lossy exporters and importers.
 	 * 
@@ -266,7 +410,7 @@ namespace
 	 * A lossy importer might skip some elements, but it is fair to require that
 	 * it imports everything which is exported by the corresponding exporter.
 	 */
-	void fuzzyCompareMaps(const Map& actual, const Map& expected)
+	void fuzzyCompareMaps(const Map& actual, const Map& expected, const QByteArray& format_id)
 	{
 		// Miscellaneous
 		QCOMPARE(actual.getScaleDenominator(), expected.getScaleDenominator());
@@ -301,9 +445,14 @@ namespace
 		QVERIFY2(actual.getNumColors() <= 2 * expected.getNumColors(), qPrintable(test_label.arg(actual.getNumColors()).arg(expected.getNumColors())));
 		
 		// Symbols
-		// Combined symbols may be dropped on export
+		// Combined symbols may be dropped (split) on export.
+		// Text symbols may be duplicated on export.
 		QVERIFY2(2 * actual.getNumSymbols() >= expected.getNumSymbols(), qPrintable(test_label.arg(actual.getNumSymbols()).arg(expected.getNumSymbols())));
 		QVERIFY2(actual.getNumSymbols() <= 2 * expected.getNumSymbols(), qPrintable(test_label.arg(actual.getNumSymbols()).arg(expected.getNumSymbols())));
+		for (auto i = 0; i < expected.getNumSymbols(); ++i)
+		{
+			fuzzyMatchSymbol(actual, format_id, expected.getSymbol(i));
+		}
 		
 		// Objects
 		QVERIFY2(actual.getNumObjects() >= expected.getNumObjects(), qPrintable(test_label.arg(actual.getNumObjects()).arg(expected.getNumObjects())));
@@ -654,6 +803,15 @@ void FileFormatTest::saveAndLoad()
 	printer_config.page_format.v_overlap += 4.0;
 	original->setPrinterConfig(printer_config);
 	
+	// Enforce a unique prefix for symbol names, allowing for reliable
+	// recognition even when truncated. However, this doesn't help with
+	// symbols which are duplicated or dropped on export/import.
+	for (auto i = 0; i < original->getNumSymbols(); ++i)
+	{
+		auto* symbol = original->getSymbol(i);
+		symbol->setName(QLatin1Char('#') + QString::number(i) + QChar::Space + symbol->getPlainTextName());
+	}
+	
 	// Save and load the map
 	auto new_map = saveAndLoadMap(*original, format);
 	QVERIFY2(new_map, "Exception while importing / exporting.");
@@ -670,7 +828,7 @@ void FileFormatTest::saveAndLoad()
 	// be independent of information which cannot be exported into this format
 	if (new_map && format->isWritingLossy())
 	{
-		fuzzyCompareMaps(*new_map, *original);
+		fuzzyCompareMaps(*new_map, *original, format_id);
 		
 		original = std::move(new_map);
 		new_map = saveAndLoadMap(*original, format);
