@@ -35,7 +35,7 @@
 #include <QByteArray>
 #include <QString>
 
-#include "util/backports.h"
+#include "util/backports.h"  // IWYU pragma: keep
 
 
 // clazy:excludeall=function-args-by-ref
